package org.deephacks.tools4j.config.internal.core.jpa;

import static org.deephacks.tools4j.config.internal.core.jpa.JpaEvents.JPA201_PROP_FILEPATH_MISSING;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import com.google.common.io.Closeables;

/**
 * EntityManagerFactoryCreator is responsible for creating an initalizing an EntityManagerFactory
 * in an standalone application-managed environment. 
 */
public class EntityManagerFactoryCreator {
    public static final String PERSISTENCE_UNIT_NAME_PARAM = "persistenceUnitName";
    public static final String JPA_PROPERTIES_FILE = "tools4j.jpa.propfilepath";

    public static final EntityManagerFactory createFactory(String persistenceUnitName) {
        Properties p = new Properties();
        String filePath = SystemProperties.createDefault().get(JPA_PROPERTIES_FILE);
        if (filePath == null || "".equals(filePath)) {
            throw JPA201_PROP_FILEPATH_MISSING(JPA_PROPERTIES_FILE);
        }
        FileInputStream in = null;
        try {
            in = new FileInputStream(new File(filePath));
            p.load(in);
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            Closeables.closeQuietly(in);
        }
        return Persistence.createEntityManagerFactory(persistenceUnitName, p);
    }

}