package org.deephacks.tools4j.config.internal.core.jpa;

import javax.persistence.EntityManagerFactory;
import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import javax.servlet.ServletRequestEvent;
import javax.servlet.ServletRequestListener;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * ServletEntityManagerLifecycle is responsible for initalizing an EntityManagerFactory
 * and make sure that EntityManagers are create and closed correctly in a servlet context.
 * 
 * This class would only be needed in a pure standalone web container.
 */
public class ServletEntityManagerLifecycle implements ServletContextListener,
        ServletRequestListener {
    public static final String PERSISTENCE_UNIT_NAME_PARAM = "tools4jPersistenceUnitName";
    public static final String JPA_PROPERTIES_FILE = "tools4j.jpa.propfilepath";
    private Logger log = LoggerFactory.getLogger(ServletEntityManagerLifecycle.class);

    @Override
    public void contextInitialized(ServletContextEvent event) {
        ServletContext sc = event.getServletContext();
        String unitName = sc.getInitParameter(PERSISTENCE_UNIT_NAME_PARAM);
        EntityManagerFactory factory = EntityManagerFactoryCreator.createFactory(unitName);
        event.getServletContext().setAttribute(PERSISTENCE_UNIT_NAME_PARAM, factory);
    }

    @Override
    public void requestInitialized(ServletRequestEvent event) {
        EntityManagerFactory factory = (EntityManagerFactory) event.getServletContext()
                .getAttribute(ServletEntityManagerLifecycle.PERSISTENCE_UNIT_NAME_PARAM);
        ThreadLocalEntityManager.createEm(factory);
    }

    @Override
    public void requestDestroyed(ServletRequestEvent sre) {
        ThreadLocalEntityManager.close();
    }

    @Override
    public void contextDestroyed(ServletContextEvent event) {
        EntityManagerFactory factory = (EntityManagerFactory) event.getServletContext()
                .getAttribute(PERSISTENCE_UNIT_NAME_PARAM);
        factory.close();
        log.info("EntityManagerFactory [{}] closed.", factory);
    }

}